#pragma once

#include "ELF.h"
#include "arch/CPU.h"
#include "arch/MMU.h"
#include "fs/VFS.h"
#include "memory/UserVM.h"
#include "thread/Thread.h"
#include <luna/LinkedList.h>
#include <luna/OwnedPtr.h>
#include <luna/Result.h>
#include <luna/Stack.h>

class Thread;

class ThreadImage
{
  public:
    static Result<OwnedPtr<ThreadImage>> try_load_from_elf(SharedPtr<VFS::Inode> inode);

    static Result<OwnedPtr<ThreadImage>> clone_from_thread(Thread* parent);

    Result<u64> push_mem_on_stack(const u8* mem, usize size);

    void apply(Thread* thread);

  private:
    OwnedPtr<UserVM> m_vm_allocator;
    PageDirectory* m_directory { nullptr };
    Stack m_user_stack;
    Stack m_kernel_stack;
    ELFData m_loaded_image_data;
    u64 m_sp;
};
